/***************************************************************************
 *   Copyright (C) 2007 by victor perez   *
 *   intirruphay@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qiconview.h>
#include <qstring.h>
#include <qptrlist.h>
#include <qcursor.h>
#include <qdragobject.h>
#include <qstringlist.h>

#include <kpopupmenu.h>
#include <klocale.h>

#include "configimagesimp.h"
#include "coremanager.h"
#include "imagemanager.h"
#include "selectdialog.h"
#include "images_core.h"
#include "imageitem.h"
#include "propertiesimp.h"
#include "vcdmanagerdata.h"
#include "resources.h"

ConfigImagesImp::ConfigImagesImp(ImageManager* manager,VcdManagerData* data,
     QWidget* parent, const char* name, bool modal, WFlags fl)
: ConfigImages(parent,name, modal,fl)
{
    a_im = manager;
    a_data = data;
    a_iconView->clear();
    initialize();
}

ConfigImagesImp::~ConfigImagesImp()
{
}

void ConfigImagesImp::iconView_dropped(QDropEvent* e,const QValueList<QIconDragItem>&)
{
    if(!QUriDrag::canDecode(e)){
        e->ignore();
        return;
    }
    QStringList lst;
    QUriDrag::decodeLocalFiles(e,lst);
    for(uint i=0;i<lst.count();++i){
        addDragFile(lst[i]);
    }
    e->acceptAction();
}

void ConfigImagesImp::iconView_mouseButtonClicked(int mouse,QIconViewItem* item,const QPoint& point)
{
    if(mouse != LeftButton&&mouse !=RightButton ) return;
    if(item == NULL)return;
    QString text = item->text();
    if(text.isEmpty())return;
    a_current = a_im->getImage( text);
    a_curItemView=item;
    if(mouse==RightButton){
        KPopupMenu* pm = menuProperties();
        pm->exec(QCursor::pos());
    }else{
        slotMountImage();
    }
    a_current = 0;
    a_curItemView=0;
}

void ConfigImagesImp::slotRemove()
{
    QIconViewItem* item = a_iconView->currentItem();
    if(item == 0)return;
    QString text = item->text();
    if(text.isEmpty())return;
    if(a_im->delImage( text )){
        delete item;
    }
}

void ConfigImagesImp::slotAdd()
{
    QPixmap image0;
    QString id;
    QString cd_point;
    QString cd_type;
    QString cd_dev;
    SelectDialog *dialog= new SelectDialog(a_data, this);

    if(dialog->exec()==QDialog::Accepted){
        cd_point=a_im->createNamePointMount();
        cd_dev=a_im->getDevice();
        cd_type=a_im->getTypeImage( dialog->fileName());
    }else return;

     id = a_im->addImage(dialog->fileName(),cd_point, cd_type,cd_dev);
     //Imagen ya fue insertada
     if(id==QString::null)return;
     image0=uic_findImage("cdrom_unmount.png");
     a_iconView->insertItem(new QIconViewItem(a_iconView,tr( id ),image0));
}

void ConfigImagesImp::addDragFile( QString file)
{
    QPixmap image0;
    QString id;

    QString cd_point;
    QString cd_type;
    QString cd_dev;

    cd_point=a_im->createNamePointMount();
    cd_dev=a_im->getDevice();
    cd_type=a_im->getTypeImage( file );
     id = a_im->addImage(file,cd_point, cd_type,cd_dev);
     //Imagen ya fue insertada
     if(id==QString::null)return;
     image0=uic_findImage("cdrom_unmount.png");
     //image0=img;
     a_iconView->insertItem(new QIconViewItem(a_iconView,tr( id ),image0));
}

void ConfigImagesImp::initialize()
{
    QPtrList<ImageItem>* ims= a_im->getImages();
    QImage img;
    QPixmap image0;
    for ( QPtrList<ImageItem>::Iterator it =ims->begin(); it!=ims->end();++it){
        img=uic_findImage(nameIcon((*it)->status()));
        image0=img;
        a_iconView->insertItem( new QIconViewItem(a_iconView,
                                tr( (*it)->name()),image0));
    }
}

KPopupMenu* ConfigImagesImp::menuProperties()
{
    KPopupMenu *pm = new KPopupMenu();
    int i=1;
    pm->insertItem(i18n("&Open with FileManager"),this,SLOT(slotOpenImage()),0,i++);
    if(!a_current->isMounted()){
        pm->setItemEnabled(i-1,false);
    }
    pm->insertItem(i18n("&Mount/Umount"),this,SLOT(slotMountImage()),0,i++);
    pm->insertItem(i18n("M&odify"),this,SLOT(slotModifyImage()),0,i++);
    pm->insertItem(i18n("&Properties"),this,SLOT(slotShowProperties()),0,i++);
    return pm;
}

void ConfigImagesImp::slotShowProperties()
{
    PropertiesImp *prop = new PropertiesImp(a_current,this);
    prop->exec();
}
void ConfigImagesImp::slotMountImage()
{
    if(a_im->changeImageStatus(a_current)){
        QImage img;
        QPixmap image0;
        img=uic_findImage(nameIcon(a_current->status()));
        image0=img;
        a_curItemView->setPixmap(image0);
   }
}

void ConfigImagesImp::slotOpenImage()
{
    a_im->openPointMointImage( a_current->pointMount());
}

void ConfigImagesImp::slotModifyImage()
{
    SelectDialog* pp= new SelectDialog(a_data,a_current,this);
    if(pp->exec()==QDialog::Accepted){
        a_current->setPointMount( pp->pointMount());
        a_current->setDevice( pp->prog());
        a_current->setType( pp->type());
        a_current->setOptions(pp->option());
    }
}

QString ConfigImagesImp::nameIcon(ImageItem::Sts_Image sts)
{
    QString res;
    switch(sts){
        case ImageItem::IMG_MOUNTED:
            res="cdrom_mount.png";
            break;
        case ImageItem::IMG_CDEMU_LOADED:
            res="cdemu_loaded.png";
            break;
        case ImageItem::IMG_CDEMU_MOUNTED:
            res="cdemu_mount.png";
            break;
        default:
            res="cdrom_unmount.png";
            break;
    }
    return res;
}


#include "configimagesimp.moc"

