/***************************************************************************
 *   Copyright (C) 2007 by victor perez   *
 *   intirruphay@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qtooltip.h>

#include <kfiledialog.h>
#include <kapplication.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "configureapplet.h"
#include "vcdmanagerdata.h"
#include "coremanager.h"
#include "imagemanager.h"
#include "resources.h"
#include "imageitem.h"

ConfigureApplet::ConfigureApplet(QWidget* parent, const char* name, bool modal, WFlags fl)
: ConfigureAppletBase(parent,name, modal,fl)
{
    CoreManager* mngr = (CoreManager*)parent;
    chkEnableCdemu->setChecked(mngr->getData()->isUseCdemu());
    chkOnlyLoad->setChecked(mngr->getData()->isOnlyLoadImageCdemu());
    edDirectory->setText(mngr->getData()->directory());
    edDirectory->setReadOnly(true);
    env4CleanMode();
    QToolTip::add(edDirectory,i18n("Directory where the applet \nmanage of point mount dirs, it should \n set permissions 'rwxrwxrwx'.\nIt's enabled when none images."));
    QToolTip::add(chkEnableCdemu,i18n("If checked,use the virtual cd driver (cdemu) "));
    QToolTip::add(chkOnlyLoad,i18n("If checked, only load the image in cdemu\n  else load the image in cdemu and\n mount cdemu device in files system"));
}

ConfigureApplet::~ConfigureApplet()
{
}

void ConfigureApplet::slotInvokeHelp()
{
    kapp->invokeHelp(QString::null,QString::fromLatin1(RC_APP_NAME));
}

void ConfigureApplet::accept()
{
  CoreManager* mngr = (CoreManager*)parent();
  mngr->getData()->setUseCdemu( chkEnableCdemu->isChecked());
  mngr->getData()->setOnlyLoadImageCdemu( chkOnlyLoad->isChecked());
  mngr->getData()->setDirectory( edDirectory->text());
  QDialog::accept();
}

void ConfigureApplet::slotSelectDir()
{
    QString directory;
    directory=KFileDialog::getExistingDirectory(QString::null,this,i18n("Base Directory"));
    if(!directory.isEmpty()){
        edDirectory->setText(directory);
    }else{
        edDirectory->setText("/");
    }
}

void ConfigureApplet::env4CleanMode()
{
    CoreManager* mngr = (CoreManager*)parent();
    if(mngr->imageManager()->numberImages()<1){
        edDirectory->setEnabled(true);
        btnSelectDir->setEnabled(true);
        textLabel1->setEnabled(true);
    }else{
        edDirectory->setEnabled(false);
        btnSelectDir->setEnabled(false);
        textLabel1->setEnabled(false);
    }
}

void ConfigureApplet::slotChangeUseCdemu( int i)
{
    if(chkEnableCdemu->isChecked()){
        CoreManager* m = (CoreManager*)parent();
        if(!m->isLoadedCdemu()){
            KMessageBox::information(0,i18n("CDEMU module not loaded"));
            chkEnableCdemu->setChecked(false);
            chkOnlyLoad->setEnabled(false);
        }else
            chkOnlyLoad->setEnabled(true);
    }else{
        CoreManager* mngr = (CoreManager*)parent();
        QPtrList<ImageItem>* ims= mngr->imageManager()->getImages();
        ImageItem* i=0;
        for(QPtrList<ImageItem>::Iterator it=ims->begin();it!=ims->end();++it){
            if((*it)->status()==ImageItem::IMG_CDEMU_MOUNTED||
             (*it)->status()==ImageItem::IMG_CDEMU_LOADED){
                KMessageBox::information(0,i18n("Unable disable use cdemu,why images using it"));
                chkEnableCdemu->setChecked(true);
                return;
            }
        }
        chkOnlyLoad->setChecked(false);
        chkOnlyLoad->setEnabled(false);
    }
}
#include "configureapplet.moc"


