/***************************************************************************
 *   Copyright (C) 2007 by victor perez   *
 *   intirruphay@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qpainter.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qmessagebox.h>
#include <qfile.h>

#include <kwin.h>
#include <kglobalsettings.h>
#include <ksystemtray.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <klocale.h>


#include "coremanager.h"
#include "coremanagertip.h"
#include "configimagesimp.h"
#include "configureapplet.h"

CoreManager::CoreManager(QWidget* parent)
 : QWidget(parent)
 , DCOPObject(RC_APP_NAME)
{
   a_pixmap = KSystemTray::loadIcon(RC_APP_ICON); //kwikdisk
   a_images = new ImageManager(this);
   a_config = new KConfig(VCDMANAGERRC);
   data   = new VcdManagerData(this);
   a_mensaje=i18n("Disc's Image Manager");
   adjustSize();
   readConfig();
}


CoreManager::~CoreManager()
{
    writeConfig();
}

void CoreManager::adjustSize()
{
   resize(a_pixmap.size());
}

void CoreManager::paintEvent( QPaintEvent *)
{
   QPainter p(this);
   int x = (width() - a_pixmap.width())/2;
   int y = (height() - a_pixmap.height())/2;
   if (x<0) x = 0;
   if (y<0) y = 0;
   p.drawPixmap(x,y,a_pixmap);
   p.end();
}

void CoreManager::mousePressEvent(QMouseEvent *ev )
{
   if(ev->button()!=LeftButton && ev->button()!=RightButton)
      return;
   QPopupMenu *pm = NULL; 
   if(ev->button()==LeftButton){
      pm = a_images->popup();
   }
   if(ev->button()==RightButton)
      pm = handleRightClick();
   showPopupMenu(pm);
}

void CoreManager::showPopupMenu( QPopupMenu *menu)
{
    QSize size = menu->sizeHint(); // geometry is not valid until it's shown
    KWin::WindowInfo i = KWin::windowInfo( winId(), NET::WMGeometry );
    QRect g = i.geometry();
    QRect screen = KGlobalSettings::desktopGeometry(g.center());

    if ( g.x()-screen.x() > screen.width()/2 &&
        g.y()-screen.y() + size.height() > screen.height() )
        menu->popup(QPoint( g.x(), g.y() - size.height()));
    else
        menu->popup(QPoint( g.x() + width(), g.y() + height()));

    //      menu->exec(mapToGlobal(QPoint( width()/2, height()/2 )));
}


/*!
    \fn CoreManager::writeConfig()
 */
void CoreManager::writeConfig()
{
    a_config->setGroup("General");
    data->writeConfig( );
    a_images->writeConfig( );
}


/*!
    \fn CoreManager::readConfig()
 */
void CoreManager::readConfig()
{
    a_config->setGroup("General");
    data->readConfig( );
    a_images->readConfig( );
}


/*!
    \fn CoreManager::imageManager()
 */
ImageManager* CoreManager::imageManager()
{
    return a_images;
}

KPopupMenu* CoreManager::handleRightClick()
{
    KPopupMenu* menu = new KPopupMenu();
    menu->setTitle(RC_APP_NAME);
    menu->insertItem(i18n("Manage Images..."),this,SLOT(slotConfigurar()),0);
    menu->insertItem(i18n("Configure vcdmanager..."),this,SLOT(slotPreferences()),0);
    return menu;
}

void CoreManager::slotConfigurar()
{
    ConfigImagesImp* pf = new ConfigImagesImp(a_images,data);
    pf->exec();
}

void CoreManager::slotPreferences()
{
    ConfigureApplet* app =  new ConfigureApplet(this);
    if(app->exec()==QDialog::Accepted){
        getData()->writeConfig();
    }
}

void CoreManager::showCustomTip( QString str)
{
    //setMensaje( str);
    //CoreManagerTip* tip= new CoreManagerTip(this);
    //tip->maybeTip( pos());
    QString title=RC_APP_NAME;
    QMessageBox::information(this,title,str);
}

//DCOP Call
bool CoreManager::mountImage(QString file)
{
    if(file.isEmpty())return false;
    //if(!getData()->isUseCleanMode()) return false;
    a_images->addImage(file,
        a_images->createNamePointMount(),
        a_images->getTypeImage( file),
        a_images->getDevice()
     );
     ImageItem* it=a_images->getImage( file);
     a_images->changeImageStatus(it);
    if(it->isMounted()){
        a_images->openPointMointImage( it->pointMount());
        return true;
    }
	return false;
}

//DCOP Call
bool CoreManager::umountImage(QString file)
{
    if (file.isEmpty())return false;
    ImageItem* it =a_images->getImage( file);
    if (it==0)return false;
    a_images->changeImageStatus( it );
    if(it->isMounted()){
        return false;
    }else
        return true;
}

bool CoreManager::isLoadedCdemu()
{
    QFile file(RC_CDEMU_FILE);
    if(!file.open(IO_ReadOnly)){
        return false;
    }else{
        file.close();
        return true;
    }
}

#include "coremanager.moc"

//#include "coremanager.skel"
