/***************************************************************************
 *   Copyright (C) 2007 by victor perez   *
 *   intirruphay@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <string.h>

#include <qstringlist.h>
#include <qstring.h>
#include <qregexp.h>
#include <qdir.h>
#include <quuid.h>
#include <qfile.h>

#include <kprocess.h>
#include <klocale.h>
#include <qmessagebox.h>

#include <mntent.h>
#include <stdio.h>

#include "imagemanager.h"
#include "resources.h"
#include "processcomunicator.h"
#include "coremanager.h"
#include "vcdmanagerdata.h"

struct data_vcdmanager/*{
    char* a_prefix_dir;
    char* a_types[NUMBER_TYPES];
    char* a_devices[NUMBER_DEVICES];
}*/ data_default = {"cd",{"auto","iso9660","vfat","udf","ufs","dmg"},{"loop","cdemu"}};

ImageManager::ImageManager(CoreManager* co)
{
    a_images = new QPtrList<ImageItem>();
    a_manager=co;
    a_curImage=0;
}

ImageManager::~ImageManager()
{
}

/*!
    \fn ImageManager::popup()
 */
KPopupMenu* ImageManager::popup()
{
   a_popup = new KPopupMenu();
   a_popup->setTitle(i18n("Images"));
   connect(a_popup,SIGNAL(activated(int)),this,SLOT(slotImagen(int)));
   int i = 1;
   for (QPtrList<ImageItem>::Iterator it = a_images->begin();it!=a_images->end();++it){
       a_popup->insertItem(extractName((*it)->name()),(*it)->id().toInt());
       if(!(*it)->isMounted()){
          a_popup->setItemEnabled((*it)->id().toInt(),false);
       }
       i++;
   }
   return a_popup;
}


/*!
    \fn ImageManager::slotImagen()
 */
void ImageManager::slotImagen(int id)
{
    openPointMointImage(getImage(id)->pointMount());
}


/*!
    \fn ImageManager::writeConfig(KConfig *config)
 */
void ImageManager::writeConfig()
{
    QStringList images;
    KConfig* config = a_manager->getConfiguration();
    config->setGroup(RC_IMAGES_GROUP);
    QPtrList<ImageItem>::Iterator it=0;
    for (it=a_images->begin();it!=a_images->end(); ++it ){
        images.append((*it)->name());
    }
    config->writePathEntry(IMAGES,images);
    for (it=a_images->begin();it!=a_images->end(); ++it ){
        config->setGroup((*it)->name());
        config->writeEntry(IMG_POINT,(*it)->pointMount());
        config->writeEntry(IMG_TYPE,(*it)->type());
        config->writeEntry(IMG_STS,(*it)->isMounted());
        config->writeEntry(IMG_DEV,(*it)->device());
        config->writeEntry(IMG_CDEMU,(*it)->cdemuNumber());
        config->writeEntry(IMG_ID,(*it)->id());
    }
    config->sync();
}


/*!
    \fn ImageManager::readConfig(KConfig *config)
 */
void ImageManager::readConfig()
{
    ImageItem* item = NULL;
    KConfig* config = a_manager->getConfiguration();
    config->setGroup(RC_IMAGES_GROUP);
    QStringList list = config->readPathListEntry(IMAGES);
    for (QStringList::Iterator it=list.begin();it!=list.end();++it ){
        item = new ImageItem();
        config->setGroup(*it);
        item->setName(*it);
        item->setType(config->readEntry(IMG_TYPE));
        item->setPointMount(  config->readEntry(IMG_POINT));
        item->setDevice( config->readEntry(IMG_DEV));
        //item->setStatus(false);
        item->setCdemuNumber(config->readEntry(IMG_CDEMU));
        item->setId( config->readEntry(IMG_ID));
        a_images->append(item);
    }
    updateImages();
}

QString ImageManager::extractName( QString name)
{
    return name.section(QDir::separator(),-1);
}

QString ImageManager::addImage( QString name,QString point,QString type, QString device)
{
    ImageItem* item=0;
    if(name==QString::null||name.length()<1)return QString::null;
    item=getImage( name );
    if(item!=0)return QString::null;
    item = new ImageItem();
    item->setName( name);
    item->setPointMount( point);
    item->setStatus( ImageItem::IMG_UMOUNTED );
    item->setType( type );
    item->setDevice( device);
    item->setId(getMaxId());
    a_images->append(item);
    return name;
}
bool ImageManager::delImage( QString id)
{
    ImageItem* item = getImage(id);
    if (item == 0) return false;
    if(item->isMounted()){
        if(!umountImage( item)) return false;
    }
    if(a_images->remove(item)){
        a_manager->getConfiguration()->deleteGroup((const QString &)id);
        a_manager->getConfiguration()->sync();
        return true;
    }
    return false;
}

ImageItem* ImageManager::getImage( QString name)
{
    int i=0;
    for (QPtrList<ImageItem>::Iterator it = a_images->begin();
         it!=a_images->end();++it,++i){
        if((*it)->name().compare(name)==0){
            return a_images->at(i);
        }
    }
    return (ImageItem*)0;
}

ImageItem* ImageManager::getImage( int id)
{
    int i=0;
    for (QPtrList<ImageItem>::Iterator it = a_images->begin();
         it!=a_images->end();++it,++i){
        if((*it)->id().toInt()==id){
            return a_images->at(i);
        }
    }
    return (ImageItem*)0;
}

QString ImageManager::getMaxId( )
{
    int i=0;
    for (QPtrList<ImageItem>::Iterator it = a_images->begin();
         it!=a_images->end();++it){
        if((*it)->id().toInt()>i){
            i=(*it)->id().toInt();
        }
    }
    return QString::number(i+1);
}

bool ImageManager::changeImageStatus( ImageItem* ite)
{
    switch(ite->status()){
        case ImageItem::IMG_CDEMU_LOADED:
        case ImageItem::IMG_CDEMU_MOUNTED:
        case ImageItem::IMG_MOUNTED:
            return umountImage(ite);
        //case ImageItem::IMG_UMOUNTED:
        default:
            return mountImage(ite);
    }
}
bool ImageManager::changeImageStatus( int id)
{
    ImageItem* it = getImage( id );
    if( it==0 )return false;
    return changeImageStatus( it );
}

bool ImageManager::mountImage( ImageItem* ite )
{
    bool inMtab;
    bool inCdemu;
    QString dir;
    inMtab=checkStatus(ite,&dir);
    inCdemu=checkCdemuMounted(extractName(ite->name()),&dir);
    if(inMtab){
        ite->setPointMount(dir);
        if(inCdemu)
            ite->setStatus(ImageItem::IMG_CDEMU_MOUNTED);
        else
            ite->setStatus(ImageItem::IMG_MOUNTED);
        return inMtab;
    }

    if(!inCdemu){
        if(a_manager->getData()->isUseCdemu())
            dir=freeCdemuDevice();
        else
            dir="-1";
        if(dir.compare("-1")==0){
            ite->setDevice(data_default.a_devices[0]);
        }else{
            ite->setDevice(data_default.a_devices[1]);
            ite->setCdemuNumber(dir);
        }
    }else{
        ite->setDevice(data_default.a_devices[1]);
        ite->setCdemuNumber(dir);
    }
    if(ite->isUseCdemu()&&!inCdemu){
        mountCdemu(ite);//Verificar resultado
    }
    inCdemu=checkCdemuMounted(extractName(ite->name()),&dir);
    if(!inCdemu){
        ite->setDevice(data_default.a_devices[0]);
        ite->setCdemuNumber("-1");
    }else{
        ite->setStatus(ImageItem::IMG_CDEMU_LOADED);
        if(a_manager->getData()->isOnlyLoadImageCdemu())
            return true;
    }

    KShellProcess* op1 = new KShellProcess();
    bool result=false;
    (*op1)<<"sudo"<<"/bin/mount"<<"-t"<<ite->type();
    if(!ite->isUseCdemu()){
        (*op1)<<"-o"<<"loop"+ite->options()<<ite->name();
    }
    else//corregir para devfs
        (*op1)<<cdemuDevicePath(ite->cdemuNumber());
    (*op1)<<ite->pointMount();
    result=createDirectory(ite->pointMount());
    if(result){
        result= executeCommand( op1);
    }
    if(result){
        if(inCdemu)
            ite->setStatus( ImageItem::IMG_CDEMU_MOUNTED);
        else
            ite->setStatus( ImageItem::IMG_MOUNTED);
    }else{
        removeDirectory(ite->pointMount());
        a_manager->showCustomTip( a_bufferTemp);
    }
    return result;
}

bool ImageManager::umountImage( ImageItem* ite)
{
    bool inMtab;
    bool inCdemu;
    QString dir;
    QString drv;
    inMtab=checkStatus(ite,&dir);
    inCdemu=checkCdemuMounted(extractName(ite->name()),&drv);
    if(!inMtab){
        if(inCdemu){
            if(a_manager->getData()->isOnlyLoadImageCdemu()){
               umountCdemu(ite);
               ite->setStatus(ImageItem::IMG_UMOUNTED);
               return true; 
            }
            ite->setStatus(ImageItem::IMG_CDEMU_LOADED);
        }else
            ite->setStatus(ImageItem::IMG_UMOUNTED);
        return false;
    }
    KShellProcess* op1 = new KShellProcess();
    if(!ite->isUseCdemu()){
        (*op1)<<"sudo"<<"/bin/umount"<<ite->name();
    }else{
        (*op1)<<"sudo"<<"/bin/umount"<<cdemuDevicePath(ite->cdemuNumber());
    }
    bool result= executeCommand(op1);
    /////////////////////////////////////////////////
    if(result)
        result=removeDirectory( ite->pointMount());
    /////////////////////////////////////////////////
    if(result){
        if(ite->isUseCdemu()){
            umountCdemu(ite);
        }
        ite->setStatus( ImageItem::IMG_UMOUNTED);
    }else{
        a_manager->showCustomTip( a_bufferTemp);
    }
    return result;
}

bool ImageManager::executeCommand( KProcess* proc)
{
/*    connect(proc,SIGNAL(receivedStdout(KProcess*,char*,int)),
    this,SLOT(slotLeerDato(KProcess*,char*,int ) ));*/
    connect(proc,SIGNAL(receivedStderr(KProcess*,char*,int)),
    this,SLOT(slotLeerDato(KProcess*,char*,int ) ));
    proc->start(KShellProcess::Block,KShellProcess::Stderr);
    if(proc->normalExit()&&proc->exitStatus()==0)
        return true;
    else
        return false;
/*    ProcessComunicator *comunicator = new ProcessComunicator(proc);
    comunicator->run();
    comunicator->wait();
    if(comunicator->finished()){
        return comunicator->getResult();
    }else{
        comunicator->stoppingAll();
        comunicator->exit();
        return false;
    }*/
}

void ImageManager::openPointMointImage( QString url)
{
    KProcess proc;
    proc<<"kfmclient"<<"exec"<<url;
    proc.start(KShellProcess::Block);
}

bool ImageManager::checkStatus( ImageItem* it,QString* dir)
{
    FILE* f = setmntent("/etc/mtab","r");
    struct mntent* m=0;
    QString file = it->name();
    while((m = getmntent(f))!=0){
        if(file==m->mnt_fsname||it->pointMount()==m->mnt_dir){;
            if(dir!=0)
                *dir=dir->insert(0,m->mnt_dir);
            endmntent(f);
            return true;
        }
    }
    endmntent(f);
    return false;
}
void ImageManager::updateImages()
{
    FILE* f = setmntent("/etc/mtab","r");
    struct mntent* m=0;
    QString* dir=0;
    QString  drv;
    ImageItem::Sts_Image res=ImageItem::IMG_UMOUNTED;
    for (QPtrList<ImageItem>::Iterator it = a_images->begin();
         it!=a_images->end();++it){
        while((m = getmntent(f))!=0){
            if((*it)->name()==m->mnt_fsname||(*it)->pointMount()==m->mnt_dir){
                if(!((*it)->name()==m->mnt_fsname))
                    res=ImageItem::IMG_CDEMU_MOUNTED;
                else
                    res=ImageItem::IMG_MOUNTED;
                dir=new QString();
                dir->insert(0,m->mnt_dir);
                (*it)->setPointMount(*dir);
                fseek(f,0L,SEEK_SET);
                break;
            }
        }
        if(res==ImageItem::IMG_UMOUNTED){
            if(checkCdemuMounted(extractName((*it)->name()),&drv))
                res=ImageItem::IMG_CDEMU_LOADED;
        }
        (*it)->setStatus(res);
        res=ImageItem::IMG_UMOUNTED;
        fseek(f,0L,SEEK_SET);
    }
    endmntent(f);
}

void ImageManager::mountCdemu( ImageItem* ite)
{
    KProcess *op1 = new KProcess();
    (*op1)<<"cdemu"<<ite->cdemuNumber()<<ite->name();
    connect(op1,SIGNAL(receivedStdout(KProcess*,char*,int)),
    this,SLOT(slotLeerDato(KProcess*,char*,int ) ));
    a_bufferTemp="";
    op1->start(KShellProcess::Block,KShellProcess::AllOutput);
    delete op1;
}

void ImageManager::umountCdemu( ImageItem* ite){
    bool inCdemu;
    QString dir;
    inCdemu=checkCdemuMounted(extractName(ite->name()),&dir);
    if(!inCdemu)return ;
    KProcess* op1 = new KProcess();
    if(ite->cdemuNumber().compare("-1")==0)return;
    (*op1)<<"cdemu"<<"-u"<<ite->cdemuNumber();

    connect(op1,SIGNAL(receivedStdout(KProcess*,char*,int)),
    this,SLOT(slotLeerDato(KProcess*,char*,int ) ));
    a_bufferTemp="";
    op1->start(KShellProcess::Block,KShellProcess::AllOutput);
    //ite->setCdemuNumber("-1"); actualizar el estado
    delete op1;
}

QString ImageManager::freeCdemuDevice()
{
    QFile file("/proc/cdemu");
    QString data;
    QRegExp devInfo( "\\s+(\\d+):\\s+(\\d+)\\s+(.+)" );
    QString free="-1";
    if(file.open(IO_ReadOnly)){
        file.readLine(data,4096);//Out Head
        while(!file.atEnd()){
            file.readLine(data,4096);
            devInfo.search(data);
            if(devInfo.cap(2).compare("0")==0){
                free=devInfo.cap(1);
                break;
            }
        }
    }
    file.close();
    return free;
}

bool ImageManager::checkCdemuMounted(QString name, QString* drv)
{
    QFile file("/proc/cdemu");
    QString data;
    QRegExp devInfo( "\\s+(\\d+):\\s+(\\d+)\\s+([^\\s\\n]+)" );
    bool free=false;
    if(file.open(IO_ReadOnly)){
        file.readLine(data,4096);//Out Head
        while(!file.atEnd()){
            file.readLine(data,4096);
            devInfo.search(data);
            if(devInfo.cap(3).compare(name)==0){ //&&devInfo.cap(2).compare("1")==0){
                *drv=devInfo.cap(1);
                free=true;
                break;
            }
        }
    }
    file.close();
    if(!free)*drv="-1";
    return free;
}

void ImageManager::slotLeerDato( KProcess* ,char* b,int l)
{
    a_bufferTemp=QString::fromLatin1(b,l);
}

bool ImageManager::createDirectory( QString f)
{
    QDir dir;
    if(!dir.exists(f)){
        return dir.mkdir(f);
    }else return true;
}

bool ImageManager::removeDirectory( QString f)
{
    QDir dir;
    if(dir.exists(f)){
        return dir.rmdir(f);
    }else return false;
}

QString ImageManager::createNamePointMount()
{
    QString directory=a_manager->getData()->directory();
    directory+=QDir::separator();
    return directory+QUuid::createUuid().toString();
}

QString ImageManager::getDevice()
{
    return data_default.a_devices[0];
}

QString ImageManager::getTypeImage( QString file)
{
    KProcess pro;
    connect(&pro,SIGNAL(receivedStdout(KProcess*,char*,int)),
    this,SLOT(slotLeerDato(KProcess*,char*,int ) ));
    a_bufferTemp="";
    pro<<"file"<<"-bi"<<file;
    pro.start(KShellProcess::Block,KShellProcess::Stdout);
    if (!a_bufferTemp.isEmpty()){
        if(a_bufferTemp.find("application/x-iso9660")>-1){
            return data_default.a_types[1];
        }
    }
    pro.clearArguments();
    pro<<"file"<<"-b"<<file;
    a_bufferTemp="";
    pro.start(KShellProcess::Block,KShellProcess::Stdout);
    if(!a_bufferTemp.isEmpty()){
        if(a_bufferTemp.find("ISO 9660")>-1){
            return data_default.a_types[1];
        }
        if(a_bufferTemp.find(" FAT ")>-1){
            return data_default.a_types[2];
        }
    }
    QRegExp regx("\\.nrg$");
    if(file.contains((const QRegExp &)regx)>-1){
        return data_default.a_types[3];
    }
    return data_default.a_types[0];
}
QString ImageManager::cdemuDevicePath(QString number)
{
    QString res;
    QString prefix="/dev/cdemu";
    QDir dir("/dev/.udev");
    if(dir.exists())// is udev
        return prefix+number;
    else
        return prefix+"/"+number;
}

#include "imagemanager.moc"
