/***************************************************************************
 *   Copyright (C) 2007 by victor perez   *
 *   intirruphay@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef IMAGEMANAGER_H
#define IMAGEMANAGER_H

#include <qobject.h>
#include <qptrlist.h>
#include <qstring.h>

#include <kconfig.h>
#include <kpopupmenu.h>

#include "imageitem.h"

const int NRO_CDEMU_DEV= 7;

class KProcess;
class CoreManager;
/**
	@author victor perez <intirruphay@gmail.com>
*/
class ImageManager: public QObject {
   Q_OBJECT
public:
    ImageManager(CoreManager*);

    ~ImageManager();

    KPopupMenu* popup();
    void writeConfig();
    void readConfig();
    /**
    * @return Return id of image
    **/
    QString addImage(QString name, QString point ,QString type,QString device);
    bool delImage(QString id);
    QPtrList<ImageItem>* getImages(){return a_images;}
    ImageItem* getImage(QString name);
    ImageItem* getImage(int id);
    bool changeImageStatus(ImageItem* img);
    bool changeImageStatus(int id);
    void openPointMointImage(QString url);
    int numberImages(){return a_images->count();}
    QString createNamePointMount();
    QString getDevice();
    QString getTypeImage(QString);
protected:
    QString extractName(QString name);
    bool mountImage(ImageItem*);
    bool umountImage(ImageItem*);
    bool executeCommand(KProcess*);
    bool checkStatus(ImageItem* it,QString* dir=0);
    void mountCdemu(ImageItem*);
    void umountCdemu(ImageItem*);
    QString freeCdemuDevice();
    bool checkCdemuMounted(QString name,QString* drv);
    bool createDirectory(QString);
    bool removeDirectory(QString);
    QString getMaxId();
    void updateImages();
    QString cdemuDevicePath(QString number);
private:
    KPopupMenu          *a_popup;
    QPtrList<ImageItem> *a_images;
    ImageItem           *a_curImage;
    CoreManager         *a_manager;
    QString         a_bufferTemp;
public slots:
    void slotImagen(int);
    void slotLeerDato(KProcess*,char*,int);
};

#endif
