/**************************************************************************
 *   Copyright (C) 2007 by victor perez   *
 *   intirruphay@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qstring.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qframe.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qtooltip.h>

#include <kfiledialog.h>
#include <klocale.h>

#include "selectdialog.h"
#include "configimagesimp.h"
#include "coremanager.h"
#include "vcdmanagerdata.h"
#include "resources.h"
//Para agregar una imagen
SelectDialog::SelectDialog(VcdManagerData* co,QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    a_data = co;
    initialize();
    a_edPointMount->setEnabled(false);
    a_btnMount->setEnabled(false);
    a_cbprog->setEnabled(false);
    a_combo->setEnabled(false);
    a_edOption->setEnabled(false);
    a_lbOption->setEnabled(false);
}

//Para agregar una imagen mediante drag
SelectDialog::SelectDialog(VcdManagerData* co, QString file, QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    a_data=co;
    initialize();
    a_edFileName->setText(file);
    a_btnFile->setEnabled(false);
    a_edPointMount->setEnabled(false);
    a_btnMount->setEnabled(false);
    a_cbprog->setEnabled(false);
    a_combo->setEnabled(false);
    a_edOption->setEnabled(false);
    a_lbOption->setEnabled(false);
}

//Para modificar las propiedades de una imagen
SelectDialog::SelectDialog(VcdManagerData* co, ImageItem* it,
        QWidget* parent , const char* name , bool modal , WFlags fl  )
    : QDialog( parent, name, modal, fl )
{
    a_data=co;
    initialize();
    a_edFileName->setText(it->name());
    a_btnFile->setEnabled(false);
    a_edPointMount->setText(it->pointMount());
    int idx = getIndex(a_combo,it->type());
    if(idx>-1){a_combo->setCurrentItem(idx);}
    idx=getIndex( a_cbprog,it->device());
    if(idx>-1){a_cbprog->setCurrentItem(idx);}
    a_edPointMount->setEnabled(false);
    a_btnMount->setEnabled(false);
    a_edOption->setText(it->options());
    QToolTip::add(a_edPointMount,i18n("Image name "));
    QToolTip::add(a_combo,i18n("Select image type.\nIt common is is9660."));
    QToolTip::add(a_edOption,i18n("Standard is empty;\n or mount options (no loop)\n vg. to Nero images (.nrg) try\n\n\t,offset=307200"));//,offset=307200
}

SelectDialog::~SelectDialog()
{
}

void SelectDialog::initialize()
{
    setName("SelectImage");
    a_hlayout1= new QHBoxLayout(0,0,0,"lay1");
    a_hlayout2= new QHBoxLayout(0,0,0,"lay2");
    a_hlayout3= new QHBoxLayout(0,0,0,"lay3");
    a_hlayout4= new QHBoxLayout(0,0,0,"lay7");
    a_vlayout = new QVBoxLayout(0,0,0,"lay4");
    a_lacombo = new QHBoxLayout(0,0,0,"lay5");
    a_laprog  = new QHBoxLayout(0,0,0,"lay6");
    a_glayout = new QGridLayout(this,1,1,1,1,"gridlay");
    a_btnAccept= new QPushButton(this,"Accept");
    a_btnCancel= new QPushButton(this,"Cancel");
    a_btnFile  = new QPushButton(this,"btnImageName");
    a_btnMount = new QPushButton(this,"btnPointMount");
    a_edFileName = new QLineEdit(this,"fileName");
    a_edPointMount = new QLineEdit(this,"pointMount");
    a_edOption = new QLineEdit(this,"options");
    a_lbOption = new QLabel(this);
    a_combo = new QComboBox(this);
    a_lbcombo= new QLabel(this);
    a_cbprog = new QComboBox(this);
    a_lbprog = new QLabel(this);
    a_laprog->addWidget(a_lbprog);
    a_laprog->addWidget(a_cbprog);
    a_lacombo->addWidget(a_lbcombo);
    a_lacombo->addWidget(a_combo);
    a_hlayout1->addWidget(a_btnFile);
    a_hlayout1->addWidget(a_edFileName);
    a_hlayout2->addWidget(a_btnMount);
    a_hlayout2->addWidget(a_edPointMount);
    a_hlayout3->addWidget(a_btnAccept);
    a_hlayout3->addWidget(a_btnCancel);
    a_hlayout4->addWidget(a_lbOption);
    a_hlayout4->addWidget(a_edOption);
    a_vlayout->addLayout(a_hlayout1);
    a_vlayout->addLayout(a_hlayout2);
    a_vlayout->addLayout(a_lacombo);
    a_vlayout->addLayout(a_laprog);
    a_vlayout->addLayout(a_hlayout4);
    a_vlayout->addLayout(a_hlayout3);
    a_glayout->addLayout(a_vlayout,0,0);
    a_edFileName->setReadOnly(true);
    a_edPointMount->setReadOnly(true);
    connect(a_btnAccept,SIGNAL(clicked()),this,SLOT(accept()));
    connect(a_btnCancel,SIGNAL(clicked()),this,SLOT(reject()));
    connect(a_btnFile,SIGNAL(clicked()),this,SLOT(addFileName()));
    connect(a_btnMount,SIGNAL(clicked()),this,SLOT(addPointMount()));
    languageChange();
    resize(QSize(300,90).expandedTo(minimumSizeHint()));

    //FIXME
    a_cbprog->setHidden(true);
    a_edPointMount->setHidden(true);
    a_btnMount->setHidden(true);
    a_lbprog->setHidden(true);
    //
    clearWState(WState_Polished);
}

void SelectDialog::languageChange()
{
    setCaption(i18n("Select Image ... "));
    a_btnAccept->setText(i18n("&Accept"));
    a_btnCancel->setText(i18n("&Cancel"));
    a_btnFile->setText(i18n("&Image Name ..."));
    a_btnMount->setText(i18n("&Point Mount ..."));
    a_lbcombo->setText(i18n("Type Image"));
    a_lbprog->setText(i18n("Device to Use"));
    a_lbOption->setText(i18n("Options of mount"));
    int i;
    for (i=0;i<NUMBER_TYPES;i++)
        a_combo->insertItem(data_default.a_types[i]);
    a_cbprog->insertItem(data_default.a_devices[0]);
    if(a_data->isUseCdemu())
        a_cbprog->insertItem(data_default.a_devices[1]);
}

void SelectDialog::addFileName()
{
    QString name;
    name = KFileDialog::getOpenFileName(QString::null,i18n("*.iso *.out *.img|Common Images\n *.*|All Files"),this,i18n("Select Image"));
    if(!name.isEmpty()){
        a_edFileName->setText(name);
    }
}

void SelectDialog::addPointMount()
{
    QString directory;
    directory=KFileDialog::getExistingDirectory(QString::null,this,i18n("Point Mount"));
    if(!directory.isEmpty()){
        a_edPointMount->setText(directory);
    }
}

QString SelectDialog::fileName()
{
    return a_edFileName->text();
}

QString SelectDialog::pointMount()
{
    return a_edPointMount->text();
}

QString SelectDialog::type()
{
    return a_combo->currentText();
}

QString SelectDialog::prog()
{
    return a_cbprog->currentText();
}
QString SelectDialog::option()
{
    return a_edOption->text();
}
void SelectDialog::accept()
{
    QString txt;
    txt=a_edFileName->text();
    if(txt.isEmpty()){
        a_btnFile->setFocus();
        return;
    }

    done(QDialog::Accepted);
}
int SelectDialog::getIndex( QComboBox* cb, QString cad)
{
    int i;
    for (i=1;i<cb->count();++i){
        if(cad.compare(cb->text(i))==0){return i;}
    }
    return -1;
}


#include "selectdialog.moc"
