/***************************************************************************
 *   Copyright (C) 2007 by victor perez   *
 *   intirruphay@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


//#include <qlcdnumber.h>
#include <qframe.h>
#include <qdialog.h>
#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kmessagebox.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <kstandarddirs.h>

#include "vcdmanager.h"
#include "configureapplet.h"
#include "vcdmanagerdata.h"
#include "resources.h"

vcdmanager::vcdmanager(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
    : KPanelApplet(configFile, type, actions, parent, name)
{
   ksConfig = config();
   widget = new CoreManager(this);
   a_tip = new CoreManagerTip(widget);
   centerWidget();
   widget->show();
   installEventFilter(KickerTip::the());
   writeDesktopEntry();
}


vcdmanager::~vcdmanager()
{
    removeDesktopEntry();
}


void vcdmanager::about()
{
    KAboutData data(RC_APP_NAME,
                I18N_NOOP("vcdmanager: An Image Disc Manager Applet"),
                "0.5",
                I18N_NOOP("Image disc manager"),
                KAboutData::License_GPL_V2,
                I18N_NOOP("(c) 2007, Víctor Pérez"));
    data.addAuthor(I18N_NOOP("Víctor Pérez"),
            i18n("Initial Version"),
            I18N_NOOP("http://www.geocities.com/intirruphay"));
    KAboutApplication dialog(&data);
    dialog.exec();
}


void vcdmanager::help()
{
    kapp->invokeHelp(QString::null,QString::fromLatin1(RC_APP_NAME));
}


void vcdmanager::preferences()
{
    ConfigureApplet* app =  new ConfigureApplet(widget);
    if(app->exec()==QDialog::Accepted){
        widget->getData()->writeConfig();
    }
}

int vcdmanager::widthForHeight(int ) const
{
    return 20;
}

int vcdmanager::heightForWidth(int ) const
{
    return height();
}

void vcdmanager::resizeEvent(QResizeEvent *e)
{
   widget->adjustSize();
   KPanelApplet::resizeEvent(e);
   centerWidget();
}

void vcdmanager::centerWidget()
{
   int x = (width() - widget->width())/2;
   int y = (height() - widget->height())/2;
   widget->move(x,y);
}

void vcdmanager::updateKickerTip(KickerTip::Data& data)
{
	QString mensajote=RC_APP_NAME;
	data.message = mensajote;
	data.subtext = widget->mensaje();
	data.icon = DesktopIcon(RC_APP_ICON,KIcon::SizeMedium);
	data.direction=popupDirection();
	data.duration=4000;
}

bool vcdmanager::writeDesktopEntry()
{
    QFile f(getFileDesktopEntry());
    QTextStream buffer(&f);
    if(f.open(IO_WriteOnly)){
        buffer<<I18N_NOOP("[Desktop Entry]\n");
        buffer<<I18N_NOOP("ServiceTypes=application/x-iso,inode/ISO-image\n");
        buffer<<I18N_NOOP("Actions=vcdmanager_add_image;\n");
        buffer<<I18N_NOOP("Encoding=UTF-8\n");
        //buffer<<"X-KDE-Submenu=vcdimager\n";
        buffer<<I18N_NOOP("[Desktop Action vcdmanager_add_image]\n");
        buffer<<I18N_NOOP("Name=Manage Image with vcdmanager\n");
        buffer<<I18N_NOOP("Exec=dcop kicker vcdmanager mountImage %u\n");
        buffer<<I18N_NOOP("Icon=")<<RC_APP_ICON;
        f.flush();
        f.close();
        return true;
    }
    return false;
}
bool vcdmanager::removeDesktopEntry()
{
    QFile f(getFileDesktopEntry());
    if(f.remove()){
        return true;
    }
    return false;
}
QString vcdmanager::getFileDesktopEntry()
{
    QString file;
    file.append(I18N_NOOP("konqueror"));
    file.append(QDir::separator());
    file.append(I18N_NOOP("servicemenus"));
    file.append(QDir::separator());
    file.append(RC_APP_NAME);
    file.append(I18N_NOOP(".desktop"));
    QString res=locateLocal(I18N_NOOP("data"),file);
    return res;
}

#include "vcdmanager.moc"

extern "C"
{
    KPanelApplet* init( QWidget *parent, const QString& configFile)
    {
        KGlobal::locale()->insertCatalogue(RC_APP_NAME);
        return new vcdmanager(configFile, KPanelApplet::Normal,
        KPanelApplet::About | KPanelApplet::Help|KPanelApplet::Preferences,
                             parent, RC_APP_NAME);
    }
}
